#import <Cocoa/Cocoa.h>

/*
 ------ EXIF image rotation ------
 
 Quick version: to be assured of respecting EXIF orientation:
 - call orientationOfImageAtPath:
 - pass the result to scaleDownToMaxWidth:...orientation:
 (Even if you don't need to scale. Can be inefficient.)
 
 The scaleDownToMaxWidth:...orientation: will rotate the image manually to allow for when NSImage doesn't do it.
 
NSImage support for orientation is weird. NSImage supports it in 10.6, but does some weird linking detection to decide whether to actually use it. The upshot is that, for example, it doesn't happen in the agent once the agent is built, but when running it unbuilt (ie. using the java executable) it does happen.
 
 So, knowing whether it's not being done automatically - and therefore whether to do it manually - is difficult. NSAppKitVersionNumber doesn't help - it always returns the 10.6 version, even in cases where NSImage ecides not to rotate.
 
 Current solution (in orientationIsRespected) is to decide by loading a known, EXIF-orientated image from the framework bundle and see whether it's rotated. Ugly, but it works.
 
 
 ------ ColorSync profiles in image files: ------
 
 These can take up lots of space (eg. 500k!) and so are a trap when shrinking images.
 Initial testing shows that shrinking images using these messages *doesn't* include the profile, so that's good.
 Haven't tested this extensively though.
 */

// Constants for EXIF orientations values. The names are chosen for what must be done to the image to make it the right way up.
typedef enum {
	NSImageOrientationNone = 1,
	NSImageOrientation90 = 6,
	NSImageOrientation180 = 3,
	NSImageOrientation270 = 8
} NSImageOrientation;

@interface NSImage (ARExtensions)

+ (NSImage*) imageReferencingFile:(NSString*) path;
+ (NSImage *) imageWithIconFamily:(IconFamilyHandle) fam;
+ (NSImage *) imageWithIcon:(IconRef) icon;

+ (NSImageOrientation) orientationOfImageAtPath:(NSString*) path;

// This shortcut is here particularly for Java, so we don't have to hard code the image name constant
+ (NSImage*) imageNamedComputer;

// We need to know the EXIF orientation to deal with the case where it's not respected and it affects the result
- (BOOL) landscape:(NSImageOrientation) ori;


// Returns a scaled-down version of this image.
// If you want to allow for exif orientation, find it out first using orientationOfImageAtPath and pass it to the second variant.
- (NSBitmapImageRep *) scaleDownToMaxWidth:(int) maxWidth maxHeight:(int) maxHeight pad:(BOOL) pad;
- (NSBitmapImageRep *) scaleDownToMaxWidth:(int) maxWidth maxHeight:(int) maxHeight pad:(BOOL) pad orientation:(NSImageOrientation) orientation;

// Returns representation that's closest to the specified width. A rep that's wider that size will always be chosen ahead of one that's narrower.
- (NSBitmapImageRep *) closestRepresentationToWidth:(int) size;

- (NSBitmapImageRep*) firstBitmap;

@end


@interface NSImageRep (ARExtensions)

// Returns a scaled-down version of this bitmap.
// If you want to allow for exif orientation, find it out first using orientationOfImageAtPath and pass it to the second variant.
- (NSBitmapImageRep *) scaleDownToMaxWidth:(int) maxWidth maxHeight:(int) maxHeight pad:(BOOL) pad;
- (NSBitmapImageRep *) scaleDownToMaxWidth:(int) maxWidth maxHeight:(int) maxHeight pad:(BOOL) pad orientation:(NSImageOrientation) orientation;

- (BOOL) isNoLargerThanWidth:(int) maxWidth height:(int) maxHeight;

@end



@interface NSBitmapImageRep (ARExtensions)

- (NSData*) convertToJpeg:(float) quality;
- (NSData*) convertToPng;
- (NSData*) convertToGif;

/*
 Creates a bitmap of the specified size, makes it transparent it and makes it the current graphics context, where subsequent
 drawing will happen.
 
 Don't forget to call restoreGraphicsContext when finished!
 
 Note, this works without a window context on 10.5+. Creating an NSImage and calling lockFocus fails in 10.5
 with no window context, so this is better.
 */
+ (NSBitmapImageRep*) makeTemporaryGraphicsContextWithWidth:(int) width height:(int) height;
+ (void) restoreGraphicsContext;

@end
